[settings]
os=baremetal
arch=xtensalx6
compiler=gcc
compiler.version=8
compiler.cppstd={{ config.project.cxxstandard }}
compiler.libcxx=libstdc++11
build_type={{ config.profiles[profile].cmake_build_type }}

[buildenv]
SCARGO_BUILD_TARGET=esp32
WORKAROUND_FOR_ESP32_C_FLAGS={{ config.project.cflags if config.project.cflags}} {{config.profiles.get(profile).cflags if config.profiles.get(profile).cflags}}
WORKAROUND_FOR_ESP32_CXX_FLAGS={{ config.project.cxxflags if config.project.cxxflags }} {{ config.profiles.get(profile).cxxflags if config.profiles.get(profile).cxxflags }}

[conf]
tools.cmake.cmaketoolchain:generator=Ninja
{% if config.project.max_build_jobs != None %}
tools.build:jobs={{config.project.max_build_jobs}}
{% endif %}
