/**
 * @copyright Copyright (C) 2023 <Company-Name>. All rights reserved.
 */

#include <iostream>
#include "{{ lib_name }}.h"

using namespace spyro;

{{ class_name }}::{{class_name}}()
{
    std::cout << "Hello from lib1" << std::endl;
}
